using System;

public delegate void TimeChangedEventHandler(
	object aSender, TimeChangedEventArgs aEventArgs);

public class TimeChangedEventArgs : EventArgs
{
	private int mTicksLeft;

	public TimeChangedEventArgs(int aTicksLeft) 
	{
		mTicksLeft = aTicksLeft;
	}
	
	public int TicksLeft 
	{
		get 
		{
			return mTicksLeft;
		}
	}

}

public class Timer
{
	private int mTickCount;
	private int mInterval;

	public event TimeChangedEventHandler TimeChanged;

	public Timer(int aTickCount, int aInterval) 
	{
		mTickCount = aTickCount;
		mInterval = aInterval;
	}

	public int TickCount 
	{
		get 
		{
			return mTickCount;
		}
	}

	public int Interval 
	{
		get 
		{
			return mInterval;
		}
	}

	protected void OnTimeChanged(int aTick) 
	{
		if (TimeChanged != null) 
		{
			TimeChangedEventArgs args = 
				new TimeChangedEventArgs(aTick);
			TimeChanged(this, args);
		}
	}

	public void run() 
	{
		int tick = mTickCount;
		while (tick > 0)
		{
			System.Threading.Thread.Sleep(mInterval);
			tick--;
			OnTimeChanged(tick);
		}
	}
}

public class TimerDemo 
{
	private static void Timer_TimeChanged(object aSender, 
		TimeChangedEventArgs aEventArgs)
	{
		Console.WriteLine("Timer! Ticks left = {0}",
			aEventArgs.TicksLeft);
	}

	public static void Main() 
	{
		Timer timer = new Timer(10, 1000);
		timer.TimeChanged += 
			new TimeChangedEventHandler(Timer_TimeChanged);
		Console.WriteLine(
			"Timer started for 10 ticks at interval 1000 ms.");
		timer.run();
	}
}